;******************************************************************************
;   This file is a basic code template for code generation on the             *
;   PIC16F916. This file contains the basic code building blocks to build     *
;   upon.                                                                     *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on             *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:        GPS COFFEE.asm                                          *
;    Date:            FEBRUARY 21, 2015                                         *
;    File Version:                                                    *
;                                                                             *
;    Author:       Ron Newton                                                 *
;    Company:                                                                 *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: P16F916.INC                                              *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes:                                                                   *
;                                                                             *
;******************************************************************************
;                                                                             *
;                                    *
;******************************************************************************


	list		p=16f916      ; list directive to define processor
	#include	<p16f916.inc> ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
;------------------------------------------------------------------------------    

;1	MCLR	NC
;2	RA0		NC	
;3	RA1		NC
;4	RA2		NC
;5	RA3		NC
;6	RA4		NC
;7	RA5		RS
;8	GND
;9	RA7		LED RED
;10	RA6		LED GREEN
;11	RC0		D0
;12	RC1		D1
;13	RC2		D2
;14	RC3		D3
;15	RC4		D4
;16	RC5		D5
;17	RC6		D6
;18	RC7		D7
;19 GND
;20	V POS
;21	RB0		E
;22	RB1		R/W
;23	RB2		NC
;24	RB3		NC
;25 RB4		NC
;26	RB5		SWITCH
;27	RB6		NC
;28	RB7		NC

	
	__CONFIG _FCMEN_ON & _IESO_ON & _CP_OFF & _WDT_OFF & _BOD_OFF & _MCLRE_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT

;------------------------------------------------------------------------------
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0,1,2, and 3.  Each Bank contains 
; Special Function Registers, General Purpose Registers, and Access RAM 
;
;------------------------------------------------------------------------------

;       Bit Equates 
;****************************************************************************
DONE    equ     7               ;done measuring flag
EMPTY   equ     5               ;flag if component is connected
V1      equ     5               ;power for C reference ckt
msb_bit equ     7               ;define for bit 7
lsb_bit equ     0               ;define for bit 0
time	equ		3
sign   equ		4

    CBLOCK 0x20 ; Define GPR variable register locations


;****************************************************************************
;       User Registers
;****************************************************************************
;       Bank 0


FLAGS  	            ;FLAGS register
count  	            ;RS232 register
txreg  	            ;RS232 data register
delay  	            ;RS232 delay register
offset  	        ;table position register
msb    	            ;general delay register
lsb    	            ;general delay register
TIMER1
TIMER2
TIMER3
SECONDS
MINUTES
HOURS
COUNT
COUNT1
PULSE
PULSE1


;		DOG REGISTERS
DATA1
COUNTER0
COUNTER1
ONES
TENS
HUNDREDS
THOUSANDS
DIGIT_1
DIGIT_2
DIGIT_3
DIGIT_4
DIGIT_5



;       Math related registers
ACCaHI  	           ;24-Bit accumulator a
ACCaMID 	
ACCaLO  	
ACCbHI  	           ;24-Bit accumulator b
ACCbMID 	
ACCbLO  	
ACCcHI  	           ;24-Bit accumulator c
ACCcMID 	
ACCcLO  	
ACCdHI  	           ;24-Bit accumulator d
ACCdMID 	
ACCdLO  	
TENK					;10,000
temp   	           		;temporary storage
    ENDC

W_TEMP         EQU    0x7D        ; w register for context saving (ACCESS)
STATUS_TEMP    EQU    0x7E        ; status used for context saving (ACCESS)
PCLATH_TEMP    EQU    0x7F        ; context saving of PC (ACCESS) 


#DEFINE		RW					PORTB,1		;RW OUT
#DEFINE		RS					PORTA,5		;RS OUT
#DEFINE		E					PORTB,0		;E OUT
#DEFINE		PROGRAM_SWITCH		PORTB,5
#DEFINE		RED					PORTA,7
#DEFINE		GREEN				PORTA,6
;------------------------------------------------------------------------------
; EEPROM INITIALIZATION
;
; The 16F916 has 256 bytes of non-volatile EEPROM, STARTing at address 0x2100
; 
;------------------------------------------------------------------------------

DATAEE    ORG  0x2100
;		DE	0x00, 0x01, 0x02, 0x03


;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

    ORG     0x0000            ; processor reset vector
    PAGESEL init
        goto    init

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------


	org 4



    ; Interrupt context saving
	BANKSEL	 PORTA			  ;change banks
    MOVWF    W_TEMP           ; save off current W register contents
    MOVF     STATUS,W         ; move status register into W register
    MOVWF    STATUS_TEMP      ; save off contents of STATUS register
    MOVF     PCLATH,W         ; move pclath register into w register
    MOVWF    PCLATH_TEMP      ; save off contents of PCLATH register

;------------------------------------------------------------------------------


INT0
		DECFSZ	  	TIMER1,1
		GOTO		INT1	
		MOVLW	    .246			;CORRECTION FOR TIME OF SECONDS
		MOVWF		TIMER1
		INCF		SECONDS,1		;INCREASE SECONDS
		MOVLW		.60
		SUBWF		SECONDS,0		;CHECK FOR 60 SECONDS
		BTFSS		3,0
		GOTO		INT1
		CLRF		SECONDS			;CLEAR SECONDS
		INCF		MINUTES,1		;INCREASE MINUTES
		MOVLW		.60				;CHECK FOR 60 MINUTES
		SUBWF		MINUTES,0
		BTFSS		3,0
		GOTO		INT1
		CLRF		MINUTES			;CLEAR MINUTES
		INCF		HOURS,1			;INCREASE HOURS

		MOVLW		.1				;CHECK FOR ONE HOUR
		SUBWF		HOURS,0
		BTFSC		3,0
		GOTO		INTA	
		MOVLW		B'00000001'
		MOVWF		FLAGS
		
INTA	MOVLW		.2				;CHECK FOR TWO HOURS
		SUBWF		HOURS,0
		BTFSS		3,2
		GOTO		INTB
		MOVLW		B'00000010'
		MOVWF		FLAGS

INTB	MOVLW		.3				;CHECK FOR 3 HOURS
		SUBWF		HOURS,0
		BTFSS		3,2
		GOTO		INTC
		MOVLW		B'00000100'
		MOVWF		FLAGS

INTC	MOVLW		.4				;CHECK FOR 4 HOURS
		SUBWF		HOURS,0
		BTFSS		3,2
		GOTO		INTD
		MOVLW		B'00001000'
		MOVWF		FLAGS

INTD	MOVLW		.8				;CHECK FOR 8 HOURS
		SUBWF		HOURS,0
		BTFSS		3,2
		GOTO		INT1
		MOVLW		B'00010000'
		MOVWF		FLAGS
	
INT1
		MOVLW	.3				;CORRECTION FOR TIME
		MOVWF	TMR0
        BCF   	INTCON,T0IF     ;Service Timer 0 overflow
		BCF		INTCON,RBIE
		BCF		INTCON,RBIF
;------------------------------------------------------------------------------
INT2
    ; Interrupt context restoring
    MOVF    PCLATH_TEMP,W     ; retrieve copy of PCLATH register
    MOVWF   PCLATH            ; restore pre-isr PCLATH register contents
    MOVF    STATUS_TEMP,W     ; retrieve copy of STATUS register
    MOVWF   STATUS            ; restore pre-isr STATUS register contents
    SWAPF   W_TEMP,F
    SWAPF   W_TEMP,W          ; restore pre-isr W register contents
    RETFIE                    ; return from interrupt
    
;------------------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------------------

init
        Banksel	PORTA			;change banks
        clrf    PORTA           ;clear PORTA, PORTB and PORTC
        clrf    PORTB
		clrf	PORTC
		BANKSEL	CMCON0			;CLEAR CMCON0
		CLRF	CMCON0
		BANKSEL	ANSEL
		CLRF	ANSEL			;CLEAR ANSEL
		BANKSEL	OPTION_REG		;change banks
        movlw  B'00010011'      ;TMR0 prescalar 1:16
        movwf   OPTION_REG
		BANKSEL	TRISA			;change banks
        MOVLW	B'00000000'
        MOVWF	TRISA			;MAKE TRIS ALL OUTPUTS
		MOVLW	B'00100000'
		MOVWF	TRISB			;MAKE PORT 5 INPUT FOR SWITCH
		CLRF	TRISC			;PORTC all outputs
		BANKSEL	WPUB
		BSF		WPUB,5			;USE PULL UP FOR SWITCH
		BSF		IOCB,5			;ALLOW INTERRUPT
        BANKSEL	LCDCON
        CLRF	LCDCON			;CLEAR LCD CON
        BANKSEL	PORTA			;change banks
		movlw	b'11100000'		;INTURUPT ON TMRO
		movwf	INTCON
		CLRF	FLAGS			;CLEAR ALL FLAGS
		BSF		FLAGS,0			;SET FLAG FOR FRESH
		CLRF	SECONDS			;CLEAR TIMERS
		CLRF	MINUTES
		CLRF	HOURS
		CLRF	INTCON			;CLEARS INTCON
		MOVLW	.100
		MOVWF	PULSE			;LOADING PULSE
		CALL	DOG				;INTIALIZE LCD


;********************TEST AREA*********************
TEST


;**************************************************
START	
		CALL	DOG				;INTIALIZE LCD
		MOVLW	0X20			;SPACE
		CALL	SEND
		MOVLW	050H			;PUSH BUTTON TO START
		CALL	SEND
		MOVLW	055H			;U
		CALL	SEND
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	048H			;H
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	042H			;B
		CALL	SEND
		MOVLW	055H			;U
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	04FH			;O
		CALL	SEND
		MOVLW	04EH			;N
		CALL	SEND
		MOVLW	020H			;SPACE
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	04FH			;0
		CALL	SEND

		MOVLW		B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF		DATA1
		CALL		WRINS_CHK								; SECOND LINE

		MOVLW	0X20			;MAKING SPACES
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND				
		MOVLW	053H			;S
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND
		MOVLW	041H			;A
		CALL	SEND
		MOVLW	052H			;R
		CALL	SEND
		MOVLW	054H			;T
		CALL	SEND

		BANKSEL	PORTA				;
		NOP
		
		BTFSS	PROGRAM_SWITCH		;ANTIBOUNCE
		GOTO	$-1
		BTFSC	PROGRAM_SWITCH
		GOTO	$-1
		MOVLW	B'11100000'			;INTURUPT ON TMRO
		MOVWF	INTCON		

S1		BTFSC	PROGRAM_SWITCH		;CHECK FOR BUTTON POSITION
		GOTO	S2
		MOVLW	.15
		MOVWF	TIMER3				;TIMER FOR HOLDING BUTTON
SA		BTFSC	PROGRAM_SWITCH
		GOTO	S2
		DECFSZ	TIMER1,1
		GOTO	SA
		DECFSZ	TIMER2,1
		GOTO	SA
		DECFSZ	TIMER3,1
		GOTO	SA
		GOTO	0X00				;RESET

;***************************READING FLAGS *********************
S2		BTFSS	FLAGS,0
		GOTO	$+2
		CALL	FRESH
		BTFSS	FLAGS,1
		GOTO	$+2
		CALL	MODERATE
		BTFSS	FLAGS,2
		GOTO	$+2
		CALL	STALE
		BTFSS	FLAGS,3
		GOTO	$+2
		CALL	ROTTEN
		BTFSS	FLAGS,4
		GOTO	S1

		CLRF	HOURS			;RESETTING TIMERS
		CLRF	MINUTES
		CLRF	SECONDS
		CLRF	FLAGS
		CALL	DOG				;INITIALIZE LCD
		MOVLW	0X20			;MAKE SPACE
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND	
		MOVLW	0X20	
		CALL	SEND	
		MOVLW	0X20	
		CALL	SEND						
		MOVLW	0X53			;SLEEPING
		CALL	SEND
		MOVLW	0X4C			;L
		CALL	SEND
		MOVLW	0X45			;E
		CALL	SEND
		MOVLW	0X45			;E
		CALL	SEND
		MOVLW	0X50			;P
		CALL	SEND
		MOVLW	0X49			;I
		CALL	SEND
		MOVLW	0X4E			;N
		CALL	SEND
		MOVLW	0X47			;G
		CALL	SEND
		MOVLW	B'11001000'		;STOP TMRO INTERRUPT WAKEUP ON PORTB CHANGE
		MOVWF	INTCON
		SLEEP
		GOTO	0X00			;START ALL OVER AGAIN

	
FRESH	CALL	DOG
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND					
		MOVLW	0X46			;FRESH
		CALL	SEND
		MOVLW	0X52			;R
		CALL	SEND
		MOVLW	0X45			;E
		CALL	SEND
		MOVLW	0X53			;S
		CALL	SEND	
		MOVLW	0X48			;H
		CALL	SEND
		CALL	TIME
		DECFSZ	PULSE,1
		RETURN
		BCF		GREEN
		BSF		RED				;TURN ON GREEN
		MOVLW	.255
		MOVWF	COUNT1
		DECFSZ	COUNT,1
		GOTO	$-1
		DECFSZ	COUNT1,1
		GOTO	$-3
		BCF		GREEN
		BCF		RED
		MOVLW	.100
		MOVWF	PULSE
		RETURN

MODERATE
		CALL	DOG
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND				
		MOVLW	0X4D			;MODERATE
		CALL	SEND
		MOVLW	0X4F			;0
		CALL	SEND
		MOVLW	0X44			;D
		CALL	SEND
		MOVLW	0X45			;E
		CALL	SEND	
		MOVLW	0X52			;R
		CALL	SEND
		MOVLW	0X41			;A
		CALL	SEND
		MOVLW	0X54			;T
		CALL	SEND
		MOVLW	0X45			;E
		CALL	SEND
		CALL	TIME
		DECFSZ	PULSE,1
		RETURN
		MOVLW	.100
		MOVWF	COUNT1
M1		BCF		RED
		BSF		GREEN			;MAKE YELLOW
		BSF		RED
		BCF		GREEN
		DECFSZ	COUNT,1
		GOTO	M1
		DECFSZ	COUNT1,1
		GOTO	M1
		BCF		GREEN
		BCF		RED
		MOVLW	.100
		MOVWF	PULSE		
		RETURN


STALE	CALL	DOG
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND		
		MOVLW	0X53			;STALE
		CALL	SEND
		MOVLW	0X54			;T
		CALL	SEND
		MOVLW	0X41			;A
		CALL	SEND
		MOVLW	0X4C			;L
		CALL	SEND	
		MOVLW	0X45			;E
		CALL	SEND
		CALL	TIME
		DECFSZ	PULSE,1
		RETURN
		BSF		GREEN
		BCF		RED				;TURN ON RED
		MOVLW	.128
		MOVWF	COUNT1
		DECFSZ	COUNT,1
		GOTO	$-1
		DECFSZ	COUNT1,1
		GOTO	$-3
		BCF		GREEN
		BCF		RED
		MOVLW	.100
		MOVWF	PULSE		
		RETURN

ROTTEN	CALL	DOG
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND				
		MOVLW	0X52			;ROTTEN
		CALL	SEND
		MOVLW	0X4F			;O
		CALL	SEND
		MOVLW	0X54			;T
		CALL	SEND
		MOVLW	0X54			;T
		CALL	SEND	
		MOVLW	0X45			;E
		CALL	SEND
		MOVLW	0X4E			;N
		CALL	SEND
		CALL	TIME
		DECFSZ	PULSE,1
		RETURN
		BSF		GREEN
		BCF		RED				;TURN ON RED
		MOVLW	.128
		MOVWF	COUNT1
		DECFSZ	COUNT,1
		GOTO	$-1
		DECFSZ	COUNT1,1
		GOTO	$-3
		BCF		GREEN
		BCF		RED
		MOVLW	.100
		MOVWF	PULSE		
		RETURN		
		RETURN


TIME
		MOVLW		B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL	CLEAR				;CLEAR REGISTERS
		MOVF	HOURS,0
		MOVWF	ACCcLO
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20	
		CALL	SEND
		MOVLW	0X20		
		CALL	SEND				
		MOVF	DIGIT_2,0		;TENS OF HOURS
		CALL	SEND
		MOVF	DIGIT_1,0		;ONES OF HOURS
		CALL	SEND
		MOVLW	0X3A			;COLON
		CALL	SEND
		CALL	CLEAR			;CLEAR REGISTERS
		MOVF	MINUTES,0
		MOVWF	ACCcLO
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVF	DIGIT_2,0		;TENS OF MINUTES
		CALL	SEND
		MOVF	DIGIT_1,0		;ONES OF MINUTES
		CALL	SEND
		MOVLW	0X3A			;COLON
		CALL	SEND
		CALL	CLEAR			;CLEAR REGISTERS
		MOVF	SECONDS,0
		MOVWF	ACCcLO
		CALL	DECIMAL
		CALL	DISPLAYING
		MOVF	DIGIT_2,0		;TENS OF SECONDS
		CALL	SEND
		MOVF	DIGIT_1,0		;ONES OF SECONDS
		CALL	SEND
		RETURN		


;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Delay Routines                                                     -*
;*--------------------------------------------------------------------------*
;****************************************************************************
vlong   movlw   0xff            ;very long delay, approx 200msec
        movwf   msb
        goto    d1
delay20                         ;20 msec delay
        movlw   .26
        movwf   msb
        goto    d1
delay16                         ;16 msec delay
        movlw   .21
        movwf   msb
        goto    d1
delay1                          ;approx 750nsec delay
        movlw   .1      
        movwf   msb
d1      movlw   0xff
        movwf   lsb
d2      decfsz  lsb,F
        goto    d2
        decfsz  msb,F
        goto    d1
        return
	

;---------------------------CONVERSION FROM HEXADECIMAL TO DECIMAL---------------------------------------------------
;*		USES	ACCcLO AND ACCcMID
;*		RESULTS IN TENK (TENS OF THOUSANDS) THOUSANDS, HUNDREDS,TENS AND ONES
;*----------------------------------------------------------------------------------
CLEAR	
		CLRF	ACCcMID
		CLRF	ACCcLO
		RETURN

DECIMAL											
		BANKSEL		PORTA			;change banks
		SWAPF		ACCcMID,W
		ANDLW		0FH
		ADDLW		0F0H
		MOVWF		THOUSANDS
		ADDWF		THOUSANDS,F
		ADDLW		0E2H
		MOVWF		HUNDREDS
		ADDLW		32H
		MOVWF		ONES

		MOVF		ACCcMID,0
		ANDLW		0FH
		ADDWF		HUNDREDS,F
		ADDWF		HUNDREDS,F
		ADDWF		ONES,F
		ADDLW		0E9H
		MOVWF		TENS
		ADDWF		TENS,F
		ADDWF		TENS,F

		SWAPF		ACCcLO,W
		ANDLW		0FH
		ADDWF		TENS,F
		ADDWF		ONES,F

		RLF			TENS,F
		RLF			ONES,F
		COMF		ONES,F
		RLF			ONES,F
			
		MOVF		ACCcLO,W
		ANDLW		0FH
		ADDWF		ONES,F
		RLF			THOUSANDS,F
		
		MOVLW		07H
		MOVWF		TENK

		MOVLW		0AH
LB1		ADDWF		ONES,F
		DECF		TENS,F
		BTFSS		3,0
		GOTO		LB1
LB2		ADDWF		TENS,F
		DECF		HUNDREDS,F
		BTFSS		3,0
		GOTO		LB2

LB3		ADDWF		HUNDREDS,F
		DECF		THOUSANDS,F
		BTFSS		3,0
		GOTO		LB3

LB4		ADDWF		THOUSANDS,F
		DECF		TENK,F
		BTFSS		3,0
		GOTO		LB4

		RETURN
;------------------------------------------------------------------------------
;************************DOG DELAYS FOR PROGRAMMING LCD******************
SEND	MOVWF		DATA1
		CALL		WRDATA_CHK
		CALL		DELAY30uS
		RETURN
WRDATA_CHK
		CALL		CHK_BUSY
		BSF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTC
		BCF			E
		MOVLW		.255
		MOVWF		PORTC
		RETURN
WRINS_CHK
		CALL		CHK_BUSY
WRINS_NOCHK
		BCF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTC
		BCF			E
		MOVLW		.255
		MOVWF		PORTC
		RETURN
CHK_BUSY
		BANKSEL		TRISC		;CHANGING OUTPUT TO INPUT
		MOVLW		.255
		MOVWF		TRISC		;LOAD
		BANKSEL		PORTC
		BCF			RS			;RS LOW
		BSF			RW			;RW HIGH
		BSF			E			;ENABLE HIGH
		CALL		DELAY30uS	;ADDED FOR BUSY CHECK
;		BTFSC		PORTC,7		;CHECK BUSY DOES NOT WORK
;		GOTO		$-1
		BANKSEL		TRISC	
		CLRF		TRISC		;LOAD MAKE OUTPUT
		BANKSEL		PORTA		;CHANGE BANKS
		BCF			E			;LOWER ENABLE
		RETURN					;RETURN


DELAY40MS
		MOVLW		.44
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN
DELAY30uS
		MOVLW		.11
		MOVWF		COUNTER0
		DECFSZ		COUNTER0,1
		GOTO		$-1
		RETURN
DELAY2MS
		MOVLW		3
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN

;***********************************INTIALIZING LCD DISPLAY*******************
DOG		CALL		DELAY40MS
		MOVLW		039H			;FUNCTION SET INSTRUCTION SET 0
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		014H			;SET BIAS 3.3 VOLT 014H   5 VOLT 01CH
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		078H			;CONTRAST SET ADJUSTMENT 3.3 VOLT 078H  5 VOLT 074H	
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		055H			;POWER CONTROL 3.3 VOLT 055H  5 VOLT 052H
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		06DH			;FOLLOWER CONTROL 3.3 VOLT 06DH     5 VOLT 069H
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		038H			;FUNCTION SET  INSTRUCTION SET 1
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		00CH			;DISPLAY ON CURSOR OFF
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		001H			;CLEAR DISPLAY
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		002H			;GO HOME
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		006H			;ENTRY MODE SET
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS	
		RETURN


;**************************** DISPLAYING *****************************8

DISPLAYING	
		MOVLW	0				;LOAD INTO 0 THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS
		ADDWF	ONES,0			;ADD THE OFFSET TO THE LOW ADDRESS (ADRES)
		MOVLW	5				;LOAD 3 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC	STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW	1				;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF	PCLATH
    	MOVF 	ONES,W      	 ;MOVE CONTROL INTO W
    	CALL    DISPLAY          ;GO TO LOOK UP TABLE       
		MOVWF	DIGIT_1
		CLRF	PCLATH

		MOVLW		0
		ADDWF		TENS,0
		MOVLW		5
		BTFSC		STATUS,C
		ADDLW		1
		MOVWF		PCLATH
		MOVF		TENS,0
		CALL		DISPLAY
		MOVWF		DIGIT_2
		CLRF		PCLATH

		RETURN
;******************************LOOK UP TABLE FOR NUMBERS*********************
		ORG	500
DISPLAY
		ADDWF	PCL,1				;ADD W TO PROGRAM COUNTER AND LOOK UP DATA  "." = DECIMAL #
		RETLW	B'00110000'			;0
		RETLW	B'00110001'			;1
		RETLW	B'00110010'			;2
		RETLW	B'00110011'			;3
		RETLW	B'00110100'			;4
		RETLW	B'00110101'			;5
		RETLW	B'00110110'			;6
		RETLW	B'00110111'			;7
		RETLW	B'00111000'			;8
		RETLW	B'00111001'			;9



;********************USED FOR TROUBLE SHOOTING******************
CHECK
		BANKSEL		PORTA
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		048H		;H
		CALL		SEND
		MOVLW		045H		;E
		CALL		SEND
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		04BH		;K
		CALL		SEND

				RETURN


  END